/*
 * Input: Keyboard interface
 *
 * Copyright © 1998-2001 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#ifndef __KEYBOARD_H
#define __KEYBOARD_H

/* Keyboard Port definitions */
#define KBD_DATA_PORT	0x60
#define KBD_CTRL_PORT	0x61
#define KBD_STATUS_PORT	0x64
#define KBD_CMD_PORT	0x64

/* Keyboard Status masks */
#define KBD_OBF		0x01
#define KBD_IBF		0x02
#define KBD_LOCKED	0x10
#define KBD_TRANSMIT_TO	0x20
#define KBD_RECEIVE_TO	0x40
#define KBD_PARITY_ERR	0x80

/* Keyboard Commands */
#define KBD_LED		0xED
#define KBD_SCANCODES	0xF0
#define KBD_CFG		0xF3
#define KBD_ENABLE	0xF4
#define KBD_DISABLE	0xF5
#define KBD_RESET	0xFF

#define KBD_BAT		0xAA
#define KBD_ACK		0xFA
#define KBD_RESEND	0xFE

/* Keyboard Controller Commands */
#define TEST_KBD	0xAA
#define TEST_I_KBD	0xAB
#define DISABLE_KBD	0xAD
#define ENABLE_KBD	0xAE
#define DISABLE_A20	0xDD
#define ENABLE_A20	0xDF
#define PULSE_RESET	0xFE

#ifdef __cplusplus
extern "C" {
#endif

unsigned kb_status(void);
unsigned kb_empty_buffer(void);
unsigned kb_read(void);
unsigned kb_wait(void);

#ifdef __cplusplus
}
#endif

#endif

