/*
 * IO / Port
 *
 * Copyright © 1998-2001,2004 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#ifndef __IOPORT_H
#define __IOPORT_H

#include <types.h>

typedef u16_t ioport_t;

#ifdef __cplusplus
extern "C" {
#endif

static inline u8_t inb(ioport_t port)
{
	u8_t result;
	asm("inb %w1, %b0": "=a" (result): "Nd" (port));
	return result;
}

static inline u16_t inw(ioport_t port)
{
	u16_t result;
	asm("inw %w1, %w0": "=a" (result): "Nd" (port));
	return result;
}

static inline u32_t inl(ioport_t port)
{
	u32_t result;
	asm("inl %w1, %0": "=a" (result): "Nd" (port));
	return result;
}

static inline void outb(ioport_t port, u8_t value)
{
	asm("outb %b0, %w1": : "a" (value), "Nd" (port));
}

static inline void outw(ioport_t port, u16_t value)
{
	asm("outw %w0, %w1": : "a" (value), "Nd" (port));
}

static inline void outl(ioport_t port, u32_t value)
{
	asm("outl %0, %w1": : "a" (value), "Nd" (port));
}

#ifdef __cplusplus
}
#endif

#endif

