/*
 * 2D Rasterizer
 *
 * Copyright © 1998-2001, 2005 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#ifndef __RASTER_H
#define __RASTER_H

#ifdef __cplusplus
extern "C" {
#endif

#include <types.h>

typedef struct {
	u8_t r;
	u8_t g;
	u8_t b;
	u8_t a;
} color_t;

struct raster_driver {
	char *name;
	int (*init)();
	int (*done)();
	bool (*wait_vertical_retrace)();
	void (*put_pixel)(u32_t x, u32_t y, u8_t color);
	u8_t (*get_pixel)(u32_t x, u32_t y);
	void (*fill_screen)(u8_t color);
	color_t (*get_pal_item)(int index);
	int (*put_pal_item)(int index, color_t *color);
};

extern struct raster_driver *raster_list[];

typedef struct {
	size_t width, length;
	size_t channels;
	void *data;
} image_t;

/*
 * Hardware Level Functions
 */
int raster_init();
int raster_done();
u8_t raster_get_pixel(u32_t x, u32_t y);
void raster_put_pixel(u32_t x, u32_t y, u8_t color);
void raster_fill_screen(u8_t color);
color_t raster_get_pal_item(int index);
int raster_put_pal_item(int index, color_t *color);
bool raster_wait_vertical_retrace();

/*
u32_t get_area(u32 x1, u32 y1, u32 x2, u32 y2, image_t *img);
u32_t put_area(u32 x1, u32 y1, u32 x2, u32 y2, image_t *img);
u32_t get_screen(image_t *img);
u32_t put_screen(image_t *img);
u32_t draw_line(u32 x1, u32 y1, u32 x2, u32 y2);
u32_t draw_hline(u32 x, u32 y, u32 length);
u32_t draw_vline(u32 x, u32 y, u32 length);
u32_t draw_circle(u32 x, u32 y, u32 radius, color_t color);
u32_t draw_ellipse(u32 x, u32 y, u32 a, u32 b, color_t color);
void draw_conic();
void draw_triangle();
u32_t draw_box(u32 x0, u32 y0, u32 x1, u32 y1, color_t color);
u32_t draw_rect(u32 x0, u32 y0, u32 x1, u32 y1, color_t color);

u32_t get_pal(palette_t *pal);
u32_t put_pal(palette_t *pal);
void rotate_pal(palette_t *pal);
void black_pal();
void fadeout();
void fadein(palette_t *pal);

void save_screen(ImageFile *img);
void load_screen(ImageFile *img);
*/

#ifdef __cplusplus
}
#endif

#endif

