/*
 * VGA hw
 *
 * Copyright © 2004, 2005 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#ifndef __VGA_HW_H
#define __VGA_HW_H

#include <types.h>
#include <ioport.h>

/* VGA number of registers per subsystem. */

#define VGA_CRT_NREGS	24
#define VGA_ATC_NREGS	21
#define VGA_GFX_NREGS	9
#define VGA_SEQ_NREGS	5
#define VGA_MSC_NREGS	1

/* VGA registers store type.  */

typedef struct {
	int	max_x;
	int	max_y;
	int	colors;
	u8_t	crt[VGA_CRT_NREGS];
	u8_t	atc[VGA_ATC_NREGS];
	u8_t	gfx[VGA_GFX_NREGS];
	u8_t	seq[VGA_SEQ_NREGS];
	u8_t	msc[VGA_MSC_NREGS];
} vga_regs_t;

/* VGA registers.  */

#define VGA_CRT_REG_ADDR	0x3d4
#define VGA_CRT_REG_DATA	0x3d5

#define VGA_CRT_CURSOR_POS_HI	0xe
#define VGA_CRT_CURSOR_POS_LO	0xf

#define VGA_CRT_REG_STATUS	0x3da

#define VGA_ATC_REG_ADDR	0x3c0
#define VGA_ATC_REG_DATA	0x3c1

#define VGA_MSC_REG		0x3c2

#define VGA_SEQ_REG_ADDR	0x3c4
#define VGA_SEQ_REG_DATA	0x3c5

#define VGA_DAC_REG_MASK	0x3c6
#define VGA_DAC_REG_ADDR_R	0x3c7
#define VGA_DAC_REG_ADDR_W	0x3c8
#define VGA_DAC_REG_DATA	0x3c9

#define VGA_GFX_REG_ADDR	0x3ce
#define VGA_GFX_REG_DATA	0x3cf


static inline int vga_msc_in(void)
{
	return inb(VGA_MSC_REG);
};

static inline int vga_crt_in(int a)
{
	outb(VGA_CRT_REG_ADDR, a);
	return inb(VGA_CRT_REG_DATA);
};

static inline int vga_crt_status_in(void)
{
	return inb(VGA_CRT_REG_STATUS);
};

static inline int vga_seq_in(int a)
{
	outb(VGA_SEQ_REG_ADDR, a);
	return inb(VGA_SEQ_REG_DATA);
};

static inline int vga_gfx_in(int a)
{
	outb(VGA_GFX_REG_ADDR, a);
	return inb(VGA_GFX_REG_DATA);
};

static inline int vga_atc_in(int a)
{
	vga_crt_status_in();
	outb(VGA_ATC_REG_ADDR, a);
	return inb(VGA_ATC_REG_DATA);
};

static inline void vga_msc_out(int a)
{
	outb(VGA_MSC_REG, a);
};

static inline void vga_crt_out(int a, int d)
{
	outb(VGA_CRT_REG_ADDR, a);
	outb(VGA_CRT_REG_DATA, d);
};

static inline void vga_seq_out(int a, int d)
{
	outb(VGA_SEQ_REG_ADDR, a);
	outb(VGA_SEQ_REG_DATA, d);
};

static inline void vga_gfx_out(int a, int d)
{
	outb(VGA_GFX_REG_ADDR, a);
	outb(VGA_GFX_REG_DATA, d);
};

static inline void vga_atc_out(int a, int d)
{
	vga_crt_status_in();
	outb(VGA_ATC_REG_ADDR, a);
	outb(VGA_ATC_REG_ADDR, d);
};

static inline void vga_atc_screen(int a)
{
	vga_crt_status_in();
	outb(VGA_ATC_REG_ADDR, a);
};

/*
inline void vga_dac_in(int a, int *r, int *g, int *b);
inline void vga_dac_out(int a, int r, int g, int b);
*/

#endif

