;
; 3D OS Kernel entry code
;
; Copyright © 2004 Guillem Jover <guillem@hadrons.org>
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software Foundation,
; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
;

#define ASM
#include <boot/multiboot.h>

    bits 32

    global entry_point
    extern kfreebsd_main
    extern multiboot_main
    extern puts

    section .multiboot

;%define MULTIBOOT_FLAGS MULTIBOOT_VIDEO_MODE
%define MULTIBOOT_FLAGS 0

multiboot:
.magic		dd	MULTIBOOT_MAGIC
.flags		dd	MULTIBOOT_FLAGS
.checksum	dd	0 - MULTIBOOT_MAGIC - MULTIBOOT_FLAGS
.addr_header	dd	0
.addr_load	dd	0
.addr_load_end	dd	0
.addr_bss_end	dd	0
.entry_addr	dd	0
.video_mode	dd	MULTIBOOT_VIDEO_MODE_GRAPHIC
.video_width	dd	640
.video_height	dd	480
.video_depth	dd	24

    section .text

entry_point:

	; This is here for the loader to verify the entry point is valid.
	dd	0x90909090

	cmp	eax, MULTIBOOT_BOOTLOADER_MAGIC
	je	multiboot_ok

    ; Test for a FreeBSD loader
	; Old disk boot blocks:
	; (*entry_point)(howto, bootdev, cyloffset, esym);
	cmp	[esp], dword 0
	je	no_boot
	; Old diskless netboot code:
	; (*entry_point)(0, 0, 0, 0, &nfsdiskless, 0, 0, 0);
	; New uniform boot code:
	; (*entry_point)(howto, bootdev, 0, 0, 0, &bootinfo)
	cmp	[esp+20], dword 0
	jne	no_boot

	add	sp, 4
	call	kfreebsd_main

	jmp	halt

    multiboot_ok:
	push	dword msg_system_boot
	call	puts
	add	sp, 4

	; Push MultiBoot info structure
	push	ebx
	call	multiboot_main
	add	sp, 4

	jmp	halt

    no_boot:
	push	msg_noboot
	call	puts
	add	sp, 4
	jmp	halt

    panic:
	push	msg_panic
	call	puts
	add	sp, 4

    halt:
	push	msg_halt
	call	puts
	add	sp, 4
    .loop:
	jmp	.loop

    section .rodata

msg_system_boot	db '<3Dkernel>v0.1 booted ...', 0
msg_halt	db 'System halted.', 0
msg_panic	db 'System panic, halting...', 0
msg_noboot	db 'error: not booted by a multiboot or kfreebsd loader.', 0

