/*
 * Main test function
 *
 * Copyright © 2004, 2005 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#include <console.h>
#include <raster/raster.h>
#include <system.h>

int
main(void)
{
	int i;
	color_t color_black = { .r = 0, .g = 0, .b = 0, .a = 0 };
	color_t color_yellow = { .r = 0x3f, .g = 0x3f, .b = 0, .a = 0 };
	color_t color_red = { .r = 0x3f, .g = 0, .b = 0, .a = 0 };
	color_t color_green = { .r = 0, .g = 0x3f, .b = 0, .a = 0 };
	color_t color_blue = { .r = 0, .g = 0, .b = 0x3f, .a = 0 };

	puts("Starting C module ...");

#if 0
	console_detect();
	memory_detect();
	pci_detect();
	pnp_detect();
	serial_detect();

	parallel_detect();
	joystick_detect();
	floppy_detect();

	ide_detect();
#endif

	raster_init();

	raster_put_pal_item(0, &color_black);
	raster_put_pal_item(1, &color_yellow);
	raster_put_pal_item(2, &color_red);
	raster_put_pal_item(3, &color_green);
	raster_put_pal_item(4, &color_blue);
	raster_fill_screen(1);

	for (i = 0; i < 100; i++) {
		raster_put_pixel(i, 10, 2);
		raster_put_pixel(i, 11, 3);
		raster_put_pixel(i, 12, 4);
	}

#if 0
	puts("Stopping C module ...");
#endif

	reboot();

	return 0;
}

#include <boot/multiboot.h>

int
multiboot_main(struct multiboot_info *mbi)
{
	(void)mbi;

	main();

	return 0;
}

#include <boot/kfreebsd.h>

int
kfreebsd_modules(u32_t *modulep)
{
	u32_t type, size;

	do {
		type = *(modulep++);
		size = *(modulep++);

		switch (type) {
		case MODINFO_NAME:
			printk("module name: %s\n", (char *)modulep);
			break;
		case MODINFO_TYPE:
			printk("module type: %s\n", (char *)modulep);
			break;
		case MODINFO_ARGS:
			printk("module args: %s\n", (char *)modulep);
			break;
		case MODINFO_ADDR:
			printk("module addr: %x\n", *modulep);
			break;
		case MODINFO_SIZE:
			printk("module size: %n\n", *modulep);
			break;
		case MODINFO_METADATA:
			printk("module metadata\n");
			break;
		case MODINFO_END:
			printk("module list end\n");
			break;
		}

		modulep += (size + sizeof(u32_t) - 1) / sizeof(u32_t);

	} while (type != MODINFO_END);

	return 0;
}

int
kfreebsd_main(u32_t howto, u32_t bootdev, int a, int b, int c,
              struct bootinfo *bi)
{
	int i;
	char *envp;

	(void)a;
	(void)b;
	(void)c;

	if (bi->bi_version != BOOTINFO_VERSION)
		puts("error: wrong kfreebsd loader version");

	printk("version: %n\n", bi->bi_version);
	printk("howto(%n) bootdev(%n)\n", howto, bootdev);
	printk("kernel: %s\n", bi->bi_kernelname);
	printk("diskless: %n\n", bi->bi_nfs_diskless);
	printk("n bios used: %n\n", bi->bi_n_bios_used);
	printk("bios_geom: ");
	for (i = 0; i < N_BIOS_GEOM; i++)
		printk("%x ", bi->bi_bios_geom[i]);
	printk("\n");
	printk("size: %n bytes\n", bi->bi_size);
	printk("memsize valid: %n\n", bi->bi_memsizes_valid);
	printk("bios dev: %n\n", bi->bi_bios_dev);
	printk("pad1(%n) pad2(%n)\n", bi->bi_pad[0], bi->bi_pad[1]);
	printk("basemem: %n KiB | extmem: %n MiB\n", bi->bi_basemem,
	       bi->bi_extmem);
	printk("symtab: %x - %x\n", bi->bi_symtab, bi->bi_esymtab);
	printk("kernend: %x\n", bi->bi_kernend);
	printk("envp: %x\n", bi->bi_envp);
	envp = (char *)bi->bi_envp;
	while (*envp != '\0') {
		printk("  %s\n", envp);
		while (*envp != '\0')
			envp++;
		(*envp)++;
	}

	printk("modulep: %x\n", bi->bi_modulep);
	if (bi->bi_modulep)
		kfreebsd_modules((u32_t *)bi->bi_modulep);

	reboot();

	return 0;
}


