/*
 * Generic Raster
 *
 * Copyright © 2005 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#include <raster/raster.h>

static struct raster_driver *raster;

static color_t color_black = {
	.r = 0,
	.g = 0,
	.b = 0,
	.a = 0,
};

int
raster_init(void)
{
	raster = raster_list[0];
	raster->init();

	return 0;
}

int
raster_done(void)
{
	int ret = 0;

	if (raster->done)
		ret = raster->done();

	raster = NULL;

	return ret;
}

void
raster_put_pixel(u32_t x, u32_t y, u8_t color)
{
	if (raster->put_pixel)
		raster->put_pixel(x, y, color);
}

u8_t
raster_get_pixel(u32_t x, u32_t y)
{
	if (raster->get_pixel)
		return raster->get_pixel(x, y);
	else
		return 0;
}

bool
raster_wait_vertical_retrace(void)
{
	if (raster->wait_vertical_retrace)
		return raster->wait_vertical_retrace();
	else
		return false;
}

void
raster_fill_screen(u8_t color)
{
	if (raster->fill_screen)
		raster->fill_screen(color);
}

color_t
raster_get_pal_item(int index)
{
	if (raster->get_pal_item)
		return raster->get_pal_item(index);
	else
		return color_black;
}

int
raster_put_pal_item(int index, color_t *color)
{
	if (raster->put_pal_item)
		return raster->put_pal_item(index, color);
	else
		return 0;
}

