/*
 * VGA text console
 *
 * Copyright © 2004, 2005 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#include <types.h>
#include <ioport.h>
#include <console.h>
#include <raster/vga.h>

/* FIXME: move that to an include file */
#define VGA_TEXT_MEM_ADDR	(void *)0xb8000

#define VGA_TEXT_DEFAULT_COLOR	0xf

typedef struct video_char {
	u8_t ch;
	u8_t color;
} video_char_t;

static const video_char_t blank_char = {
	.ch = ' ',
	.color = VGA_TEXT_DEFAULT_COLOR,
};

static video_char_t *video;
static int video_size;
static int p;
static int max_x, max_y;
static int color;
static int cursor;

static int vga_text_cursor_get_pos(void);

static int
vga_text_init(void)
{
	max_x = 80;
	max_y = 25;
	color = VGA_TEXT_DEFAULT_COLOR;
	video_size = max_x * max_y;
	video = VGA_TEXT_MEM_ADDR;
	p = vga_text_cursor_get_pos();
	cursor = 1;

	return 0;
}

static void
vga_text_cursor_set_pos(int pos)
{
	vga_crt_out(VGA_CRT_CURSOR_POS_HI, (pos >> 8) & 0xff);
	vga_crt_out(VGA_CRT_CURSOR_POS_LO, pos & 0xff);
}

static int
vga_text_cursor_get_pos(void)
{
	int pos;

	pos = vga_crt_in(VGA_CRT_CURSOR_POS_HI) << 8;
	pos |= vga_crt_in(VGA_CRT_CURSOR_POS_LO);

	return pos;
}

static int
vga_text_scroll_up(void)
{
	video_char_t *src, *dst;
	int i;

	src = video + max_x;
	dst = video;

	for (i = 0; i < video_size - max_x; i++)
		dst[i] = src[i];
	for (; i < video_size; i++)
		dst[i] = blank_char;

	p -= max_x;

	return i;
}

static int
vga_text_putchar(const int c)
{
	switch (c) {
	case '\r':
		p += max_x;
		break;
	case '\n':
		p += (max_x) - (p % max_x);
		break;
	case '\t':
		p += TABSIZE;
		break;
	default:
		if (p >= video_size) {
			vga_text_scroll_up();
		}
		video[p].ch = c;
		video[p].color = color;
		p++;
		break;
	}

	vga_text_cursor_set_pos(p);

	return 0;
}

static struct console_driver vga_text_cons = {
	.name = "vga_text",
	.init = vga_text_init,
	.putchar = vga_text_putchar,
};

struct console_driver *cons_list[] = {
	&vga_text_cons,
	NULL,
};

