;
; 3D OS Boot code
;
; Copyright © 1998-2001 Guillem Jover <guillem@hadrons.org>
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software Foundation,
; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
;

#include <boot/boot.h>

    bits 16
    org 0

	jmp	short boot_ini

	id	db '<3Dboot>v0.1 ', 0

    boot_ini:
	mov	ax, BOOT_SEGMENT
	mov	ds, ax
	mov	ax, INIT_SEGMENT
	mov	es, ax
	xor	si, si
	xor	di, di
	mov	cx, 0x100
	rep	movsw
	jmp	INIT_SEGMENT:ini

    ini:

	mov	ax, LOADER_SEGMENT	; Initialize:
	mov	es, ax			;  ES at LOADER_SEGMENT
	cli				;  SS at LOADER_SEGMENT:SP at 0x3ffe
	mov	ss, ax			;  DS at INIT_SEGMENT
	mov	sp, STACK_TOP - 2
	sti
	mov	ax, INIT_SEGMENT
	mov	ds, ax
	mov	si, id
	call	puts

	xor	ah, ah			; reset FDC
	xor	dl, dl
	int	0x13

    load_loader:
	mov	al, '.'
	call	putc
	xor	bx, bx
	mov	cx, 2
	xor	dx, dx
	mov	ah, 2
	mov	al, LOADER_SECTORS
	int	0x13
	jc	load_loader		; redundant redundancy ?

	call	putnl

	jmp	LOADER_SEGMENT:0

    halt:
	mov	si, msg_halt
	call	puts

    hang:
	jmp	hang


; function: puts(unsigned char *);
; input:
;    [DS:SI] -> ASCIZ to write

puts:
	push	ax
	push	bx
.print:	lodsb
	or	al, al
	je	.fi
	mov	ah, 0xE
	mov	bx, 0x0007
	int	0x10
	jmp	short .print
.fi:
	pop	bx
	pop	ax
	ret

; function: putw(unsigned);
; input:
;    DX -> word to write

putw:
	mov	cx, 4
putb:
	push	cx
	mov	cl, 4
	rol	dx, cl
	pop	cx
	mov	ax, 0x0e0f
	and	al, dl
	add	al, 0x90
	daa
	add	al, 0x40
	daa
	int	0x10
	loop	putb
	ret

; function: putnl(void);

putnl:
	mov	al, 0xD
	call	putc
	mov	al, 0xA

; function: putc(unsigned char);
; input:
;     AL -> Char to write

putc:
	mov	ah, 0xE
	int	0x10
	ret

msg_halt	db 'System halted...', 0

	times 510 - ($ - $$) db 0
	db 0x55, 0xAA

