;
; CPU detection routines for 3D OS
;
; Copyright © 1998-2001 Guillem Jover <guillem@hadrons.org>
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software Foundation,
; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
;


;
; CPU families
;

#define CPUf_unknown	-1
#define CPUf_8088	0
#define CPUf_188	1
#define CPUf_286	2
#define CPUf_386	3
#define CPUf_486	4
#define CPUf_Pentium	5
#define CPUf_P6		6
#define CPUf_Pentium4	7

;
; Intel CPU models
;

#define CPUm_8088	0
#define CPUm_V20	2
#define CPUm_V40	4
#define CPUm_8086	1
#define CPUm_V30	3
#define CPUm_V50	5

#define CPUm_188	0
#define CPUm_186	1

#define CPUm_286	0

#define CPUm_386DX	0
#define CPUm_386SX	2
#define CPUm_386CX	2
#define CPUm_386EX	2
#define CPUm_386SL	4
#define CPUm_376	3

#define CPUm_486DX	0
/*#define CPUm_486DX	1*/
#define CPUm_486SX	2
#define CPUm_486DX2	3
#define CPUm_486	4
#define CPUm_486SX2	5
#define CPUm_486WBEDX2	7	/* Write-Back Enhanced DX2 */
#define CPUm_486DX4	8
/*
#define CPUm_486DXL	3
#define CPUm_486DXL2	3
#define CPUm_486SL	4
#define CPUm_486SL2	8
#define CPUm_486SLC	9
#define CPUm_486SLC2	10
#define CPUm_486DLC	11
#define CPUm_486DLC2	12
*/

#define CPUm_P5		0
#define CPUm_P5T	1
#define CPUm_P54C	2
#define CPUm_P54T	2
#define CPUm_P54M	2
#define CPUm_P24T	3
#define CPUm_P55C	4
#define CPUm_P55CTP	4

#define CPUm_P6		0
#define CPUm_PentiumPro	1
#define CPUm_P6L	3
#define CPUm_PentiumII	3
#define CPUm_Klamath	3
#define CPUm_Katmai	4
#define CPUm_Willamette	5
#define CPUm_Deschutes	6
#define CPUm_Mendocino
#define CPUm_Celeron	7
#define CPUm_PentiumIII	7

#define CPUm_Pentium4	7

;
; AMD CPU models
;

#define CPUm_K6		
#define CPUm_K6_2	
#define CPUm_K6_III	
#define CPUm_Duron	
#define CPUm_Athlon	
#define CPUm_Thunderbird	
#define CPUm_Palomino	

;
; CPU brands
;

#define CPUb_Intel	0
#define CPUb_AMD	1
#define CPUb_Cyrix	2
#define CPUb_NexGen	3
#define CPUb_IBM	4
#define CPUb_Nec	5
#define CPUb_Sony	6
#define CPUb_UMC	7
#define CPUb_CT		8
#define CPUb_TI		9
#define CPUb_RapidCad	10
#define CPUb_Centaur	11
#define CPUb_Rise	11

;
; NPU families
;

#define NPUf_onchip	-3
#define NPUf_no		-2
#define NPUf_emu	-1
#define NPUf_8088	0
#define NPUf_80188	1
#define NPUf_80286	2
#define NPUf_80386	3
#define NPUf_80486	4
#define NPUf_Pentium	5
#define NPUf_P6		6
#define NPUf_Pentium4	6


#define CPUfamily	byte [es:CPU_FAMILY]
#define CPUmodel	byte [es:CPU_MODEL]
#define CPUstepping	byte [es:CPU_STEPPING]
#define CPUbrand	byte [es:CPU_BRAND]
#define CPUtype		byte [es:CPU_TYPE]
#define CPUfeatures	dword [es:CPU_FEATURES]
#define NPUfamily	byte [es:NPU_FAMILY]



#define FLAGS16		word [bp - 4]
#define FLAGS32		dword [bp - 4]

#define CPUname_offs	18
#define CPUname_end	byte [bp - CPUname_offs - 12]	/* 0 for ASCIZ */
#define CPUname(x)	dword [bp - CPUname_offs + x]	/* 12 bytes */

cpu_detect:
	push	bp
	mov	bp, sp
	sub	sp, 30

	mov	si, msg_cpu_test
	call	puts

	mov	CPUname_end, 0

	mov	CPUfamily, CPUf_8088
	mov	CPUmodel, CPUm_8088
	mov	CPUbrand, CPUb_Intel

	pushf
	pop	ax
	mov	FLAGS16, ax		; Save 16 bit Flags

	mov	ax, 1			; 88 shift using all bits of CL
	mov	cl, 33			; 188+ shift using only low 5 bits
	shl	ax, cl			; of CL
	cmp	ax, 0
	jnz	.m188

.m88:
	mov	ax, FLAGS16
	and	ax, 0x0fff		; 88/188 & clones bits 12-15 set to 1
	push	ax
	popf
	pushf
	pop	ax
	and	ax, 0xf000
	jz	.mUnknown

	clc
	pusha				; executed as jmp $+2 on 8088/86
	stc				; on NECs as pusha
	jc .mV20
.PQ:					; prefetch queue test code
	xor	dh, dh
	mov	dl, CPUmodel
	mov	al, 0x90		; x88 has 4 bytes prefetch queue
	mov	byte [cs:.patch], 0x42	; x86 has 6 bytes prefetch queue
	mov	cl, 32			; INC DX -> OpCode 0x42
	rol	ah, cl			; NOP    -> OpCode 0x90
	mov	[cs:.patch], al
	nop
	nop
	nop
	nop
.patch:	inc	dx
	mov	CPUmodel, dl
	jmp	.end

.mV20:
	popa
	mov	CPUmodel, CPUm_V20
	mov	CPUbrand, CPUb_Nec
	jmp	.PQ

.mUnknown:
	mov	CPUfamily, CPUf_unknown
	jmp	.end

.m188:
	mov	CPUfamily, CPUf_188
	mov	CPUmodel, CPUm_188

	push	sp			; 188- dec sp then push value
	pop	ax			; 286+ push value then dec sp
	cmp	ax, sp
	jnz	.PQ			; sp != ax then 188-

.m286:
	mov	CPUfamily, CPUf_286
	mov	CPUmodel, CPUm_286

	mov	ax, FLAGS16
	or	ax, 0xf000		; 286 cannot set bits 12-15 in RM
	push	ax
	popf
	pushf
	pop	ax
	and	ax, 0xf000
	jnz	.m386maybe		; bits 12-15 = 0 then 286
	jmp	.end

.m386maybe:
	test	ax, 0x8000		; 386+ cannot set bit 15
	jz	.m386			;  maybe 286 can set bit 15?

	mov	si, msg_286_bit15
	call	puts
	jmp	.end

.m386:
	mov	CPUfamily, CPUf_386
	mov	CPUmodel, CPUm_386DX
	pushfd
	pop	ebx
	mov	FLAGS32, ebx
	or	ebx, 0x40000		; 486+ can set AC flag
	push	ebx
	popfd
	pushfd
	pop	ebx
	test	ebx, 0x40000
	jnz	.m486
	jmp	.end			; Is not a 486

.m486:
	mov	CPUfamily, CPUf_486
	mov	CPUmodel, CPUm_486DX
	or	ebx, 0x200000		; Pentium+ can set ID flag
	push	ebx
	in	al, 0xe8		; Cyrix CPU need to enable ID flag
	and	al, 0x80		; CCR4 reg bit7
	out	0xe8, al
	popfd
	pushfd
	pop	ebx
	test	ebx, 0x200000
	jnz	.mPentium
	jmp	.end			; Is not a Pentium

.mPentium:
	mov	CPUfamily, CPUf_Pentium
	mov	CPUmodel, CPUm_P5
	xor	eax, eax		; CPU brand
	cpuid
	mov	CPUname(0), ebx
	mov	CPUname(4), edx
	mov	CPUname(8), ecx

	mov	si, bp
	sub	si, CPUname_offs
	call	puts
	mov	al, ' '
	call	putc

	mov	eax, 1			; CPU type & cfg
	cpuid
	mov	CPUstepping, al
	and	CPUstepping, 0x0f
	shr	al, 4
	and	al, 0x0f
	mov	CPUmodel, al
	mov	CPUfamily, ah
	and	CPUfamily, 0x0f
	shr	ah, 4
	and	ah, 0x0f
	mov	CPUtype, ah
	mov	CPUfeatures, edx

.end:

	xor	bh, bh
	mov	bl, CPUfamily
	shl	bx, 1
	mov	si, [cpu_family_str + bx]
	call	puts
	mov	si, msg_cpu_model
	call	puts
	mov	dl, CPUmodel
	call	putb
	mov	si, msg_cpu_step
	call	puts
	mov	dl, CPUstepping
	call	putb

	cmp	CPUfamily, CPUf_386	; Check for 32 bit processors
	jb	.no32bit
	push	dword FLAGS32
	popfd
	mov	si, msg_cpu_Ok
	call	puts
	jmp	.npu

.no32bit:
	push	FLAGS16
	popf
	mov	si, msg_cpu_Bad
	call	puts
	jmp	.npu
.npu:
;	mov	si, msg_npu
;	call	puts

	mov	sp, bp
	pop	bp
	ret



cpu_family_str:
	dw cpu_m8088_str, cpu_m188_str, cpu_m286_str, cpu_m386_str,
	dw cpu_m486_str, cpu_mPentium_str, cpu_mP6_str, cpu_mPentium4_str, 0

cpu_model_str:
cpu_m8088_str		db '8088', 0
cpu_m8086_str		db '8086', 0
cpu_mV20_str		db 'V20', 0
cpu_mV30_str		db 'V30', 0
cpu_m188_str		db '188', 0
cpu_m186_str		db '186', 0
cpu_m286_str		db '286', 0
cpu_m386_str		db '386', 0
cpu_m486_str		db '486', 0
cpu_mPentium_str	db 'Pentium', 0
cpu_mP6_str		db 'P6', 0
cpu_mP6L_str		db 'Pentium II', 0
cpu_mPentium4_str	db 'Pentium 4', 0

cpu_brand_str:
	dw cpu_bIntel_str, cpu_bAMD_str, cpu_bCyrix_str, cpu_bUMC_str,
	dw cpu_bNexGen_str, cpu_bCentaur_str, cpu_bRise_str, 0

cpu_bIntel_str		db 'GenuineIntel', 0
cpu_bAMD_str		db 'AuthenticAMD', 0
cpu_bCyrix_str		db 'CyrixInstead', 0
cpu_bUMC_str		db 'UMC UMC UMC ', 0
cpu_bNexGen_str		db 'NexGenDriven', 0
cpu_bCentaur_str	db 'CentaurHauls', 0
cpu_bRise_str		db 'RiseRiseRise', 0


msg_cpu_test	db 'cpu type: ', 0
msg_cpu_model	db ' model: ', 0
msg_cpu_step	db ' stepping: ', 0
msg_cpu_Ok	db ' Ok', 0xd, 0xa, 0
msg_cpu_Bad	db ' error 32bit CPU needed', 0xd, 0xa, 0
msg_cpu_unknown	db 'unknown', 0
msg_286_bit15	db ' 286 bit 15 set ', 0




%define delay	times 8 jmp $ + 2

%macro clock_init 0
	mov	al, 0xb4
	out	0x43, al
	xor	ax, ax
	out	0x42, al
	out	0x42, al
%endmacro

%macro clock_end 0
	mov	al, 0x80
	out	0x43, al
	in	al, 0x42
	mov	dl, al
	in	al, 0x42
	mov	dh, al
	neg	dx
%endmacro


#define NINSTR		2000
#define TICKS		word [bp - 2]
#define Hi_TSC		dword [bp - 6]
#define Lo_TSC		dword [bp - 10]


speed_detect:
	push	bp
	mov	bp, sp
	sub	sp, 4

	mov	si, msg_speed_test
	call	puts

	cmp	CPUfamily, CPUf_Pentium
	jb	.aam_code
	
	test	CPUfeatures, 0x10	; is TSC bit set (bit 4)
	jz	.aam_code

	rdtsc
	mov	Lo_TSC, eax
	mov	Hi_TSC, edx

	; wait some time

	rdtsc
	sub	Lo_TSC, eax
	sbb	Hi_TSC, edx

	; compute MHz

;	jmp .aam_code_end

.aam_code:
	push	es			; construct the code to test speed
	mov	ax, ds			;  AAM ninstr + 1 RETN in the stack
	mov	es, ax
	mov	di, STACK_TOP
	mov	ax, [speed_instr]
	mov	cx, NINSTR
	cld
	rep	stosw
	mov	al, 0xC3		;  RETN -> OpCode 0xC3
	stosb
	pop	es

	cli
	in	al, 0x61
	push	ax
	and	al, 11111101b		; disable speaker
	or	al, 1			; enable timer 2
	out	0x61, al

	clock_init
	call	STACK_TOP
	clock_end
	mov	TICKS, dx

	pop	ax			; restore speaker status
	out	0x61, al
	sti

	mov	di, aam_ticks
	xor	bh, bh
	mov	bl, CPUfamily
	mov	bl, [di + bx]		; Load AAM ticks related to CPU
	mov	ax, 2386		; Magic number :->
	imul	bx			;  Hz = (nr. instr * cpu cycles instr *
	xor	dx, dx			;   * clock Hz) / exec clock ticks
	div	TICKS			;  2000 ins * 1193181 Hz / 1 MHz = 2386
	call	putdec

	mov	si, msg_speed
	call	puts

	mov	ax, TICKS		; To view instruction exec clock ticks
	call	putdec

	mov	si, msg_speed_end
	call	puts

	mov	sp, bp
	pop	bp
	ret


speed_instr	dw 0x0AD4		; AAM instruction
aam_ticks	db 83, 19, 16, 17, 15, 18, 15, 0

msg_speed_test	db 'cpu speed: ', 0
msg_speed	db ' MHz (clock ticks = ', 0
msg_speed_end	db ')', 0xd, 0xa, 0

