;
; 3D OS Loader Emulation code
;
; Copyright © 1998-2001 Guillem Jover <guillem@hadrons.org>
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software Foundation,
; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
;

; DOS com executable emulating a booted session
;
; A COM executable starts its code at an offset of 0x100 and the used
; ‘org’ directive should have this value, but this emulation intends to
; go a step further, and normalizes segments at run time to fake the
; comiler/os/computer/loader.
;
; Normalization Process:
;
; In real mode, segments are left shifted by 4 bits, or multiplied
; by 0x10 when converted to physical addresses. If we want to keep
; the code intact maintaining consistency we have to do something
; that permits execution with our code situated at the beginning of a
; segment. Because COM execs start at an offset 0x100 of their segement
; we have to add 0x10 to the segment, then the references made by the
; compiler will be correct because they reference a fixed position
; with respect to their segment (offset 0).
;
; Before segment normalization we have to reference data with an added
; offset of 0x100, as the compiler starts the exec at offset 0, and the
; OS loads it at offser 0x100.
;

%macro loader_emu_init 0
	; Normalize segments for boot emulation

	mov	ax, normalized_start
	mov	[jump_addr + 0x100], ax

	mov	ax, cs
	add	ax, 0x10		; CS + 0x10 : text starts at 0
	mov	[jump_addr + 0x102], ax

	jmp	far [jump_addr + 0x100]	; Jump to appropriate code segment
					;  invalidating the prefetch queue

normalized_start:

	mov	ds, ax
	xor	si, si			; DS = CS (loader segment)
	cli
	mov	ss, ax			; SS = CS (loader segment)
	mov	sp, STACK_TOP - 2	; SS: STACK_TOP -> stack end
	sti
	mov	bx, bootsect
	shr	bx, 4
	add	ax, bx			; CS + bootsect
	mov	es, ax			; ES -> bootsect segment
	xor	di, di
%endmacro

%macro loader_emu_end 0
	mov	bx, end + 0x100		; Do the inverse process
	mov	[jump_addr], bx

	mov	bx, cs
	sub	bx, 0x10
	mov	[jump_addr + 2], bx

	jmp	far [jump_addr]
end:
	mov	ax, 0x4c00
	int	0x21

jump_addr	dd 0
		align 16
bootsect:	db 0xde, 0xad
		times 508 db 0
		db 0x55, 0xAA
%endmacro

