;
; Video Detection routines for 3D OS
;
; Copyright © 1998-2001 Guillem Jover <guillem@hadrons.org>
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software Foundation,
; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
;

video_detect:
	mov	ah, 0x12
	mov	bl, 0x10
	int	0x10
	cmp	bl, 0x10
	jne	video_detect.ega

	mov	cx, 0x8000		; HGA
	mov	dx, 0x3ba
	in	al, dx
	mov	bl, 0x80
	and	bl, al
.hga_loop:
	in	al, dx
	and	al, 0x80
	cmp	bl, al
	jne	video_detect.hga
	loop	video_detect.hga_loop
	jmp	video_detect.mda
.hga:	mov	al, VIDEO_HGA
	jmp	short video_detect.end

.mda:	mov	ah, 0x0F		; MDA PGA CGA
	int	0x10
	cmp	al, 7
	jne	video_detect.cga
	mov	al, VIDEO_MDA
	jmp	short video_detect.end

.cga:	mov	al, VIDEO_CGA
	jmp	short video_detect.end
					; EGA XGA VGA SVGA
.ega:	mov	ax, 0x1c00		; Function 0x1a not used to prevent
	mov	cx, 0x0001		; missdetection of earlier EGAs
	int	0x10
	cmp	al, 0x1c
	je	video_detect.xga
	mov	al, VIDEO_EGA
	jmp	short video_detect.end

.xga:	mov	ax, 0x1200		; XGA VGA SVGA
	mov	bl, 0x37
	int	0x10
	cmp	al, 0x12
	jne	video_detect.svga
	mov	al, VIDEO_XGA
	jmp	short video_detect.end

	; NOTE!!! boot set stack at INIT_SEGMENT:STACK_TOP-2
	;  and ES is at INITSEG

.svga:	mov	di, STACK_TOP
	mov	ax, 0x4f00
	int	0x10
	cmp	ax, 0x004f
	jne	video_detect.vga
	mov	al, VIDEO_SVGA
	jmp	short video_detect.end

.vga:	mov	al, VIDEO_VGA
.end:	mov	[es:VIDEO_TYPE], al
	ret


video_print_info:
	xor	bx, bx
	mov	bl, [es:VIDEO_TYPE]
	cmp	bx, VIDEO_LAST
	ja	.end

	mov	si, msg_video_type
	call	puts

	shl	bx, 1
	mov	si, [video_type + bx]
	call	puts

	mov	si, msg_video_type_end
	call	puts

.end:
	ret


msg_video_type		db 'console type: ', 0
msg_video_type_end	db ' Ok', 0xd, 0xa, 0

video_type:
	dw mda, hga, pga, cga, ega, mcga, xga, vga, svga

hga	db 'hercules', 0
pga	db 'pga', 0
mda	db 'mda', 0
cga	db 'cga', 0
ega	db 'ega', 0
mcga	db 'mcga', 0
xga	db 'xga', 0
vga	db 'vga', 0
svga	db 'svga', 0

card_detect:
	xor	bp, bp
;	mov
	ret

card_detect_list:
	dw ati_detect
	dw realtek_detect
	dw ahead_detect
	dw paradise_detect
	dw oak_detect
	dw genoa_detect
	dw s3_detect
	dw trident_detect
	dw 0

ati_detect:
	ret
	ati_inf		db '761295520', 0
			dw 0x31
realtek_detect:
	ret
	realtek_inf	db 'REALTEK VGA', 0
			dw 0x45
ahead_detect:
	ret
	ahead_inf	db 'AHEAD', 0
			dw 0x25
paradise_detect:
	ret
	paradise_inf	db 'VGA=', 0
			dw 0x7D
oak_detect:
	ret
	oak_inf		db 'OAK VGA', 0
			dw 0x25
genoa_detect:
	ret
	genoa_inf	db 0x77, 0, 0x99, 0x66, 0
			dw 0x37
s3_detect:
	ret

trident_detect:
	ret

